@@ -7,7 +7,7 @@ BEGIN {
 }
 
 
-our $VERSION =  '0.01_05';
+our $VERSION =  '0.02_14';
 
 =head1 NAME
 
@@ -267,6 +267,11 @@ sub bbreak {
 package    # hide the package from the PAUSE indexer
 	DB;
 
+# In theory this may break user's code because this usage cause dependencies are loaded
+# in different order under debugger
+use Scope::Cleanup qw/ establish_cleanup /;
+use Sub::Metadata qw/ mutate_sub_is_debuggable /;
+
 
 
 ## Utility subs
@@ -289,6 +294,8 @@ sub _all_frames {
 
 # This sub is called twice: at compile time and before run time of 'main' package
 sub applyOptions {
+	@DB::options{ keys %{ $_[0] } } =  values %{ $_[0] }   if @_;
+
 	# Q: is warn expected when $DB::trace == undef?
 	$DB::trace =  $DB::options{ trace_line } || 0
 		if defined $DB::options{ trace_line };
@@ -360,8 +367,11 @@ sub state {
 		}
 
 		${ "DB::$name" } =  $value;
-		$frame->{ $name } =  $value
-			unless $set_only_global;
+		unless( $set_only_global ) {
+			defined $value
+				? $frame->{ $name } =  $value
+				: delete $frame->{ $name };
+		}
 	}
 
 
@@ -507,6 +517,9 @@ BEGIN { # Initialization goes here
 	$DB::inSUB    //=  0;
 	$DB::interaction //=  0;
 	# TODO: set $DB::trace at CT
+
+	# Some configuration options may be applied when debugger is loading
+	# When debugger is loaded its &import subroutine will be called (see comment there)
 	applyOptions();
 }
 
@@ -570,7 +583,7 @@ BEGIN { # Initialization goes here
 			# Keep list of $filenames we perhaps manipulate traps
 			$DB::_tfiles->{ $filename } =  1;
 
-			*dbline =  $main::{ "_<$filename" }; #WORKAOUND (see commit)
+			*dbline =  $main::{ "_<$filename" }; #WORKRAOUND RT#119799 (see commit)
 
 			return \%{ "::_<$filename" };
 		}
@@ -598,6 +611,13 @@ BEGIN { # Initialization goes here
 
 
 
+	sub eval_cleanup {
+		DB::state( 'eval', undef );
+	}
+	mutate_sub_is_debuggable( \&eval_cleanup, 0 );
+
+
+
 	# We put code here to execute it only once
 	(my $usercontext =  <<'	CODE') =~ s#^\t\t##gm;
 		BEGIN{
@@ -615,6 +635,10 @@ BEGIN { # Initialization goes here
 		# BUG: PadWalker does not show DB::eval's lexicals
 		# Q? It is better that PadWalker return undef instead of warn when out of level
 
+		establish_cleanup \&eval_cleanup;
+		DB::state( 'eval', 1 );
+
+
 		local $^D;
 		# FIX: when we eval user's sub the main script frame is changed
 
@@ -626,7 +650,7 @@ BEGIN { # Initialization goes here
 		# local $_ =  $DB::context[4];
 
 		local @_ =  @{ $DB::context[0] };
-		eval "$usercontext; package " .state( 'package' ) .";\n$expr";
+		eval "$usercontext; package " .state( 'package' ) .";\n#line 1\n$expr";
 		#NOTICE: perl implicitly add semicolon at the end of expression
 		#HOWTO reproduce. Run command: X::X;1+2
 	}
@@ -774,8 +798,6 @@ BEGIN { # Initialization goes here
 
 
 
-use Guard;
-
 	sub scall {
 
 		# TODO: implement debugger debugging
@@ -814,22 +836,24 @@ use Guard;
 		local $ext_call      =  $ext_call +1;
 
 		# Manual localization
-		scope_guard {
+		my $scall_cleanup =  sub {
 			# $DB::single =  DB::state( 'single' );
 			DB::state( 'single', DB::state( 'single' ) );
 
 			print $DB::OUT "<< scall back $from($f:$l) <-- $sub\n"
 				if $DB::options{ ddd };
+
+			if( $DB::options{ dd } ) {
+				pop @{ DB::state( 'state' ) };
+
+				$options{ dd } =  0;
+
+				print $DB::OUT "OUT DEBUGGER  <<<<<<<<<<<<<<<<<<<<<< \n"
+					if $DB::options{ ddd };
+			}
 		};
-
-		scope_guard {
-			pop @{ DB::state( 'state' ) };
-
-			$options{ dd } =  0;
-
-			print $DB::OUT "OUT DEBUGGER  <<<<<<<<<<<<<<<<<<<<<< \n"
-				if $DB::options{ ddd };
-		}   if $DB::options{ dd };
+		mutate_sub_is_debuggable( $scall_cleanup, 0 );
+		establish_cleanup $scall_cleanup;
 
 		# TODO: testcase 'a 3 $DB::options{ dd } = 1'
 		local $ddlvl          =  $ddlvl            if $DB::options{ dd };
@@ -902,7 +926,8 @@ sub import { # NOTE: The import is called at CT yet
 
 
 	# if we set trace_load we want to see which modules are used in main::
-	# So we apply this just before main:: is compiled but after debugger is loaded
+	# It has no any effect at RT because all modules are loaded
+	# So we apply this at CT just before main:: is compiled but after debugger is loaded
 	$DB::options{ trace_load } =  $RT_options{ trace_load }
 		if defined $RT_options{ trace_load };
 
@@ -910,8 +935,14 @@ sub import { # NOTE: The import is called at CT yet
 	# The RT options are applied after 'main::' is loaded
 	$RT_options{ trace_goto } //=  1;
 
-	# FIX? should I call applyOptions here and not at trace_load
-	# What is the benefit?
+
+	# NOTICE: it is useless to set breakpoints for not compiled files
+	# TODO: spy module loading and set breakpoints
+	$DB::dbg->init();
+
+	# Now debugger and all required modules are loaded. We should set
+	# corresponding perl debugger *internal* values based on given %DB::options
+	applyOptions( \%RT_options );
 }
 
 
@@ -922,6 +953,7 @@ sub postponed {
 	}
 
 	# RT options applied after main program is loaded
+	# IT: debug BEGIN blocks of main::
 	if( $_[0] eq "*main::_<$0" ) {
 		my @keys =  keys %RT_options;
 		@DB::options{ @keys } =  @RT_options{ @keys };
@@ -933,7 +965,7 @@ sub postponed {
 
 # TODO: implement: on_enter, on_leave, on_compile
 sub DB {
-	scope_guard {
+	establish_cleanup sub {
 		print $DB::OUT "DB::state: l:$DB::ddlvl b:$DB::inDB:$DB::inSUB e:$DB::ext_call s:$DB::single t:$DB::trace\n";
 		print $DB::OUT "TRAPPED OUT: $DB::ddlvl\n";
 	}   if $DB::options{ ddd };
@@ -1047,9 +1079,9 @@ sub init {
 	state( 'file',    $f );
 	state( 'line',    $l );
 
-	# Someone may stop client's code running through perl interface
-	# For example until the first line of client's code the $DB::singe == 0
-	# When $^P & 0x20 perl set $DB::single before execution of first line
+	# Someone may stop client's code running through perl debugger interface
+	# For example until the first line of client's code the $DB::single == 0
+	# When ($^P & 0x20) perl set $DB::single = 1 before execution of first line
 	# So we should update our state
 	DB::state( 'single', $DB::single );
 
@@ -1092,7 +1124,8 @@ sub process {
 	# in __FILE__:__LINE__ context of script we are debugging
 	print $DB::OUT "No command found. Evaluating '$str'...\n"   if $DB::options{ ddd };
 	my @result =  map{ $_ // $DB::options{ undef } } DB::eval( $str );
-	@result =  ()   if $@  &&  $result[0] eq $DB::options{ undef }; #WORKAOUND (see commit)
+	@result =  ()   if $@  &&  @result
+		&&  $result[0] eq $DB::options{ undef }; #WORKAROUND (see commit)
 
 	local $" =  $DB::options{ '"' }  //  $";
 	print $DB::OUT "@result\n"   unless $quiet;
@@ -1166,7 +1199,7 @@ sub pop_frame {
 	local $ext_call =  $ext_call  +1;
 	my $last =  pop @{ DB::state( 'stack' ) };
 	print $DB::OUT "POP  FRAME <<<< l:$DB::ddlvl b:$DB::inDB:$DB::inSUB e:$DB::ext_call s:$DB::single t:$DB::trace  --  $last->{ sub }\@". @{ DB::state( 'stack' ) } ."\n"
-		. "    $last->{ file }:$last->{ line } }\n\n"
+		. "    $last->{ file }:$last->{ line }\n\n"
 		if $DB::options{ ddd };
 
 	if( @{ DB::state( 'stack' ) } ) {
@@ -1188,7 +1221,7 @@ sub push_frame2 {
 		3;
 	}
 
-	print $DB::OUT "PUSH FRAME >>>>  l:$DB::ddlvl b:$DB::inDB:$DB::inSUB e:$DB::ext_call s:$DB::single t:$DB::trace  --  $DB::sub\n"
+	print $DB::OUT "PUSH FRAME $_[0] >>>>  l:$DB::ddlvl b:$DB::inDB:$DB::inSUB e:$DB::ext_call s:$DB::single t:$DB::trace  --  $DB::sub\n"
 		if $DB::options{ ddd };
 
 	if( $_[0] ne 'G' ) {
@@ -1196,11 +1229,13 @@ sub push_frame2 {
 		# WORKAROUND: for broken frame. Here we are trying to be closer to goto call
 		# Most actual info we get when we trace script step-by-step at this case
 		# those vars have sharp last opcode location.
-		my( $p, $f, $l ) =  caller 2;
-		DB::state( 'package', $p );
-		DB::state( 'file',    $f );
-		DB::state( 'line',    $l );
-		printf $DB::OUT "    cursor(PF) => $p, $f, $l\n"   if $DB::options{ ddd };
+		if( !DB::state( 'eval' ) ) {
+			my( $p, $f, $l ) =  caller 2;
+			DB::state( 'package', $p );
+			DB::state( 'file',    $f );
+			DB::state( 'line',    $l );
+			printf $DB::OUT "    cursor(PF) => $p, $f, $l\n"   if $DB::options{ ddd };
+		}
 
 		my $stack =  DB::state( 'stack' );
 		push @{ $stack }, {()
@@ -1248,11 +1283,12 @@ sub trace_returns {
 sub push_frame {
 	$ext_call++; scall( \&push_frame2, @_ );
 
-	if( $DB::options{ ddd } && 0 ) {
+	if( $DB::options{ ddd } ) {
 		print $DB::OUT "STACK:\n";
-		print $DB::OUT '    ' .$_->{ single } .' ' .$_->{ sub }
-			." -- @{ $_->{ caller } }\n"
-			for @{ DB::state( 'stack' ) };
+		for my $frame ( @{ DB::state( 'stack' ) } ) {
+			printf $DB::OUT "    %s %s -- %s:%s\n",
+				map{ $frame->{ $_ } // '' } qw/ single sub file line /;
+		}
 		print $DB::OUT "Frame created for $DB::sub\n\n";
 	}
 }
@@ -1263,7 +1299,7 @@ sub push_frame {
 sub sub {
 	$DB::_sub =  $DB::sub;
 	print $DB::OUT "DB::sub  l:$DB::ddlvl b:$DB::inDB:$DB::inSUB e:$DB::ext_call s:$DB::single t:$DB::trace  --  "
-		.sub{ "$DB::sub <-- @{[ map{ s#.*?([^/]+)$#$1# } (caller 0)[1,2] ]}" }->()
+		.sub{ "$DB::sub <-- @{[ map{ s#.*?([^/]+)$#$1#; $_ } ((caller 0)[1,2]) ]}" }->()
 		."\n"
 		if $DB::options{ ddd } && $DB::sub ne 'DB::can_break';
 
@@ -1276,7 +1312,9 @@ sub sub {
 	}
 
 	if( $DB::sub eq 'DB::pop_frame' ) {
-		DB::state( 'single', 0 )   unless $DB::options{ dd };
+		# DB::state( 'single', 0 )   unless $DB::options{ dd };
+
+		#FIX: Manage $DB::inSUB here also
 
 		BEGIN{ 'strict'->unimport( 'refs' )   if $options{ s } }
 		return &$DB::sub;
@@ -1288,7 +1326,7 @@ sub sub {
 
 	# manual localization
 	print $DB::OUT "\nCreating frame for $DB::sub\n"   if $DB::options{ ddd };
-	scope_guard \&DB::pop_frame; # This should be first because we should
+	establish_cleanup \&DB::pop_frame; # This should be first because we should
 	# start to guard frame before any external call
 
 	push_frame( 'C' );
@@ -1340,7 +1378,7 @@ sub lsub : lvalue {
 	}
 	else {
 		# manual localization
-		Hook::Scope::POST( \&sub_returns );
+		establish_cleanup \&sub_returns;
 		push @{ DB::state( 'stack' ) }, {
 			single      =>  DB::state( 'single' ),
 			sub         =>  $DB::sub,
@@ -1368,11 +1406,6 @@ sub lsub : lvalue {
 # It is better to load modules at the end of DB::
 # because of they will be visible to 'trace_load'
 use Devel::DebugHooks::Commands;
-BEGIN {
-	# NOTICE: it is useless to set breakpoints for not compiled files
-	# TODO: spy module loading and set breakpoints
-	$DB::dbg->init();
-}
 
 
 
@@ -1560,3 +1593,33 @@ Breakpoint does not work for this when hash key is initialized
 #TODO: $X=(condition)
 
 The debugger do not single step into sub called from string
+
+
+
+Notice strange file:line
+POP  FRAME <<<< l:0 b:0:0 e:1 s:1 t:1  --  Apache::DB::handler@1
+    /home/kes/perl_lib/lib/perl5/x86_64-linux-gnu-thread-multi/Apache/DB.pm:77 }
+
+	else {
+		if (ref $r) {
+		$SIG{INT} = \&DB::catch;
+		$r->register_cleanup(sub {
+			$SIG{INT} = \&DB::ApacheSIGINT();
+		});
+		}
+	}
+
+    print "HERE: " .$DB::single; #line 77
+    DB::state( 'trace', 1 );
+    $DB::single = 1;
+    print "HERE: A" .$DB::single;
+
+
+  print "DONE\n";
+
+  print "DONE\n";
+    return 0;
+
+}
+
+Maybe because DESTROY is called at first OP after closing block
